package PureMessage::Group;
use strict;
use warnings;

use base 'PureMessage::DBI';

use File::Glob qw(bsd_glob);

use PureMessage::List::Utils qw(list_absfile list_key2file mkpath_dotdot);
use PureMessage::Resources::Manager;

__PACKAGE__->table(q{pmx_group});
__PACKAGE__->columns(Primary => q{group_name});
__PACKAGE__->columns(All => qw/group_name long_name/);

my $GROUP_MEMBER_RESOURCE = q{members-per-group};

# friendly wrapper to create a new group
sub add_group {
    my ($group_name, $long_name) = @_;
    die(qq{Must supply a group group_name!\n}) unless $group_name;
    die(qq{Must supply a group long_name!\n}) unless $long_name;
    die(qq{Invalid group group_name: $group_name\n})
        unless ($group_name =~ qr/^[A-Za-z0-9][\w-]+$/);
    $group_name = lc($group_name);
    die "Sorry, supergroup is reserved.\n" if $group_name eq 'supergroup';
    my $group = __PACKAGE__->search(group_name => $group_name)->first();
    die(qq{Error creating group $group_name: Group already exists!\n}) if $group;
    eval {
        $group = __PACKAGE__->create(
            { long_name => $long_name, group_name => $group_name }
        );
    };
    die(qq{Error creating group $group_name: $@\n}) if $@;

    # XXX The resource database does not allow an empty list that is part
    # XXX of a multilist to be inserted, so we only touch the file here.
    eval {
        _touch_group_file($group_name);
    };
    die(qq{Cannot touch file for group $group_name: $@\n}) if $@;

    return $group;
}

sub delete_group {
    my $group_name = shift;
    die qq{Must supply a group group_name!\n} unless $group_name;
    $group_name = lc($group_name);
    die "Sorry, supergroup is reserved.\n" if $group_name eq 'supergroup';

    my $group = __PACKAGE__->search(group_name => $group_name)->first();
    die qq{Group $group_name not found!\n}
        unless $group;

    # Delete items belonging to this group from resources.
    eval {
        _delete_group_resource($group_name);
        _delete_group_file($group_name);
    };
    die qq{Cannot delete items for group $group_name: $@\n} if $@;

    $group->delete();
}

sub _touch_group_file {
    my ($group_name) = @_;

    return unless $group_name;

    my $dir = list_absfile($GROUP_MEMBER_RESOURCE);

    die(qq{Cannot fetch directory of multilist: $GROUP_MEMBER_RESOURCE!\n}) unless $dir;

    my $full_path = list_key2file($group_name, $dir);
    die(qq{Cannot fetch file path for group with group_name: $group_name!\n})
        unless $full_path;

    die(qq{Cannot make path for file: $full_path!\n})
        if mkpath_dotdot($full_path);

    open(my $fh, q{>}, $full_path) or die(qq{Cannot touch file: $full_path!\n});
    close($fh);
}

sub _delete_group_resource {
    my ($group_name) = @_;

    return unless $group_name;

    my $manager = PureMessage::Resources::Manager->new();
    die qq{Cannot create resource manager!\n} unless $manager;

    my $resource = $manager->find($GROUP_MEMBER_RESOURCE);
    die(qq{Cannot find resource for '$GROUP_MEMBER_RESOURCE'!\n})
        unless $resource;

    my $list = $resource->native();
    die(qq{Cannot create list object for resource '$GROUP_MEMBER_RESOURCE'!\n})
        unless $list;

    my $group = $list->find_list(key => $group_name);
    die(qq{Cannot find group $group_name!\n}) unless $group;

    my $data = $group->data();

    if($data) {
        $data->clear();
        $data->commit();
    }
}

#
# Group files are a sublist of a multilist. They are stored
# PREFIX/etc/members-per-group/<first-letter>/<second-letter>/<group_name>,
# so we need to clean up the directory structure in an
# intelligent way when deleting a group file.
#
sub _delete_group_file {
    my ($group_name) = @_;

    my $dir = list_absfile($GROUP_MEMBER_RESOURCE);
    die(qq{Cannot fetch directory of multilist: $GROUP_MEMBER_RESOURCE!\n})
        unless $dir;

    my $full_path = list_key2file($group_name, $dir);
    die(qq{Cannot fetch file path for group with group_name: $group_name!\n})
        unless $full_path;

    my $listdir_deep = File::Basename::dirname($full_path);
    my $listdir_shallow = File::Basename::dirname($listdir_deep);

    if (-e $full_path) {
        unlink($full_path)
            or die(qq{Failed to delete resource file for group $group_name: $full_path\n});
    }
#    warn "Checking $listdir_deep";
    if (-d $listdir_deep and not bsd_glob(qq{$listdir_deep/*})) {
#        warn "unlinking $listdir_deep";
        rmdir($listdir_deep)
            or die(qq{Failed to delete resource directory for group $group_name: $listdir_deep: $!\n});
    }
#    else {
#        use Data::Dumper;
#        warn Dumper bsd_glob(qq{$listdir_deep/*});
#    }
    if (-d $listdir_shallow and not bsd_glob(qq{$listdir_shallow/*})) {
        rmdir($listdir_shallow)
            or die(qq{Failed to delete resource directory for group $group_name: $listdir_shallow\n});
    }
}

1;
