package PureMessage::Group::List;
use strict;
use warnings;
use Carp;
use ActiveState::Config;
use PureMessage::List;
use PureMessage::Map;
use PureMessage::MultiList;
use PureMessage::Resources::Manager;
use PureMessage::Group::PermissionItem;

use base qw{Exporter};
our @EXPORT_OK = qw{add_group_list delete_group_list rename_group_list};

sub add_group_list {
    my (%opts) = @_;
    my %match_types = (is => 1, contains => 1, matches => 1, nmatches => 1,
                       re => 1, domain => 1, mail => 1);

    croak(q{Missing required option(s)!})
        unless ($opts{id}
                    && $opts{name}
                    && $opts{description}
                    && $opts{match_type});

    croak(qq{Invalid match type: $opts{match_type}})
        unless exists($match_types{$opts{match_type}});

    croak(qq{Invalid list id: $opts{id}})
        unless ($opts{id} =~ /^[A-Za-z][\w-]+$/);

    # Make sure there is not an existing "list" with this ID
    my $id = lc $opts{id};

    # Check duplicate List ids
    my $conf = PureMessage::List->Config();
    croak(qq{A list with id '$id' already exists, please use a different ID.\n})
          if ActiveState::Config::path_node($conf->root(), "list#$id");

    # Check duplicate MulitList ids
    $conf = PureMessage::MultiList->Config();
    croak(qq{A multilist with id '$id' already exists, please use a different ID.\n})
          if ActiveState::Config::path_node($conf->root(), "multilist#$id");

    # Check duplicate Map ids
    $conf = PureMessage::Map->Config();
    croak(qq{A map with id '$id' already exists, please use a different ID.\n})
          if ActiveState::Config::path_node($conf->root(), "map#$id");

    # Default to case insensitive
    my $comparator =
        (exists($opts{case_sensitive}) && $opts{case_sensitive})
            ? q{i;octet} : q{i;ascii-casemap};

    # Define a new resource
    my $res_def = PureMessage::Resources::Manager->Define(
        name => $id,
        type => q{multilist},
        attributes => {
            name => $opts{name},
            description => $opts{description},
            match_type => $opts{match_type},
            comparator => $comparator,
            group_list => 1,
        },
    );
    croak "Cannot create resource: $id" unless $res_def;
    my $res = PureMessage::Resources::Manager->new->add($id);
    croak "Cannot create instance: $id" unless $res;

    # Add new permission item
    PureMessage::Group::PermissionItem::ReadWrite->add_permission_item(qq{configuration.lists.$id});
    warn "The local copy of multilist will be created with the next run of 'pmx-profile sync-from-db --all'\n";
}

sub delete_group_list {
    my (%opts) = @_;
    croak(q{Missing required option(s)!}) unless $opts{id};
    croak(qq{Invalid list id: $opts{id}}) unless $opts{id} =~ /^[A-Za-z][\w-]+$/;

    my $id = lc $opts{id};
    if(PureMessage::Resources::Manager->Find($id)) {
        PureMessage::Resources::Manager->DeleteResourceDefinition($id);
        PureMessage::Group::PermissionItem::ReadWrite->delete_permission_item("configuration.lists.$id");
        warn "The local copy of multilist will be removed with the next run of 'pmx-profile sync-from-db --all'\n";
    }
    else {
        croak("Cannot delete non-existing list: $id\n");
    }
}

sub rename_group_list {
    my (%opts) = @_;
    croak(q{Missing required option(s)!}) unless $opts{id} && $opts{name};
    croak(qq{Invalid list id: $opts{id}}) unless $opts{id} =~ /^[A-Za-z][\w-]+$/;
    
    my $id = lc $opts{id};
    my $res = PureMessage::Resources::Manager->new->find($id);
    croak("Cannot rename non-existing list: $id\n") unless $res;
    $res->rename($opts{name});    
}

1;
