package PureMessage::Group::PermissionItem;
use strict;
use warnings;
use base qw(PureMessage::DBI);

__PACKAGE__->table(q{permission_item});
__PACKAGE__->columns(Primary => q{permission_item});
__PACKAGE__->columns(All => qw{permission_item default_permission_bits});

{
    my %cached_perms;
    sub default_permission {
        my ($perm) = @_;
        return get_permission($perm)->default_permission_bits;
    }

    sub get_permission {
        my ($perm) = @_;
        refresh_permissions() unless %cached_perms;
        
        # Retry once if it's missing, to deal with cache consistency issues.
        refresh_permissions() unless defined($cached_perms{$perm});

        die "'$perm' is not a valid permission\n" 
            unless defined $cached_perms{$perm};
        return $cached_perms{$perm};
    }

    sub refresh_permissions {
        %cached_perms = ();
        for my $p (PureMessage::Group::PermissionItem::Binary->retrieve_all,
                   PureMessage::Group::PermissionItem::ReadWrite->retrieve_all) {
            my $perm = lc $p->permission_item;
            $cached_perms{$perm} = $p;
        }
    }

    sub permission_names {
        refresh_permissions() unless %cached_perms;
        return keys %cached_perms;
    }

    sub permission_list {
        my $regex = shift;
        refresh_permissions() unless %cached_perms;
        return [ grep { $_ =~ $regex } keys %cached_perms ];
    }
}

# friendly wrapper to create a permission item
sub add_permission_item {
    my ($class, $name) = @_;

    die(q{Cannot call a class method on an object!}) if ref($class);
    die(qq{Must supply a permission_item name!\n}) unless $name;
    $name = lc($name);

    # This should raise an exception
    eval { get_permission($name) };
    if (! $@) {
        die "Permission '$name' already exists.\n";
    }

    my $permission_item;
    eval {
        $permission_item = $class->create(
            { permission_item => $name, }
        );
    };
    die(qq{Error creating permission_item $name: $@\n}) if $@;

    refresh_permissions();

    return $permission_item;
}

sub delete_permission_item {
    my ($class, $name) = @_;

    die(q{Cannot call a class method on an object!}) if ref($class);
    die(qq{Must supply a permission_item name!\n}) unless $name;
    $name = lc($name);

    my $permission_item = $class->search({ permission_item => $name, })->first;
    die "No such permission - $name\n" unless $permission_item;

    $permission_item->delete();

    refresh_permissions();
}

package PureMessage::Group::PermissionItem::Binary;
use strict;
use warnings;
use base 'PureMessage::Group::PermissionItem';

PureMessage::Group::PermissionItem::Binary->table(q{binary_permission_item});

package PureMessage::Group::PermissionItem::ReadWrite;
use strict;
use warnings;
use base 'PureMessage::Group::PermissionItem';

PureMessage::Group::PermissionItem::ReadWrite->table(q{read_write_permission_item});

1;
