package PureMessage::GroupAdminPermission;
use strict;
use warnings;
use base qw(PureMessage::DBI);
use PureMessage::Group::PermissionItem;

__PACKAGE__->table(q{group_admin_permission});
__PACKAGE__->columns(Primary => q{group_admin_permission_id});
__PACKAGE__->columns(All => qw{group_admin_permission_id group_admin_id permission_item permission_bits});
if (__PACKAGE__->SUPER::can(q{pg_version})) {
    __PACKAGE__->sequence(q{group_admin_permission_group_admin_permission_id_seq});
}

sub can {
    my ($pkg, $group_admin, $perm) = @_;
    my $value = _get_perm($group_admin, $perm);
    return $value & 1;
}

sub can_read {
    my ($pkg, $group_admin, $perm) = @_;
    my $value = _get_perm($group_admin, $perm);
    return $value & 2 ? 1 : 0;
}

sub can_write {
    my ($pkg, $group_admin, $perm) = @_;
    my $value = _get_perm($group_admin, $perm);
    return $value & 4 ? 1 : 0;
}

sub get_permission {
    my ($ga, $perm_name) = @_;
    my $value = _get_perm($ga, $perm_name); 
    return PureMessage::Group::Permission->new($perm_name, $value);
}

sub all_permissions {
    my $ga = shift;
    my @perm_names = PureMessage::Group::PermissionItem::permission_names;
    my @perms;
    for my $perm (@perm_names) {
        push @perms, PureMessage::Group::Permission->new(
                                $perm, _get_perm($ga, $perm));
    }
    return \@perms;
}

sub set_permission {
    my ($ga, $perm, $value) = @_;
    my $perm_pkg = _perm_to_pkg($perm);
    my $bits = $perm_pkg->word_to_bits($value);
    my $p = $perm_pkg->find_or_create({group_admin_id => $ga->group_admin_id,
                                       permission_item => $perm,
                                      });
    $p->permission_bits($bits);
    $p->update;
    die "Could not create perm: $perm_pkg ($perm, $bits)" unless $p;
}

sub _perm_to_pkg {
    my ($perm) = @_;
    # find type of the default permission
    my $default_perm = PureMessage::Group::PermissionItem::get_permission($perm);
    # create a new permission
    my $base_pkg = __PACKAGE__;
    (my $perm_pkg = ref($default_perm)) =~ s/.+::(\w+)/${base_pkg}::$1/;
    return $perm_pkg;
}

sub _get_perm {
    my ($ga, $perm) = @_;
    my $value;
    # recursively look at higher level permissions to see if any of them
    # aren't allowed.  If _get_perm dies, then the permission doesn't exist
    if ($perm =~ /^(.+)\.\w+$/) {
        eval {
            $value = _get_perm($ga, $1);
        };
        # only off (0) and read (2) permissions should short circuit
        return $value if !$@ and defined $value and ($value == 0 or $value == 2);
    }
    # We're here if a higher level permission doesn't exist, OR it does exist,
    # but permission is permitted
    $value = PureMessage::Group::PermissionItem::default_permission($perm);
    my $perm_pkg = _perm_to_pkg($perm);
    my $p = $perm_pkg->search(group_admin_id => $ga->group_admin_id,
                              permission_item => $perm)->first;
    $value = $p->permission_bits if $p;
    return $value;
}

package PureMessage::GroupAdminPermission::Binary;
use strict;
use warnings;
use base 'PureMessage::GroupAdminPermission';

# Each object in this class is a row in the database holding
# a permission for a group/admin pair

__PACKAGE__->table(q{group_admin_binary_permission});

sub word_to_bits {
    my ($pkg, $word) = @_;
    return 0 if $word eq 'off';
    return 1 if $word eq 'on';
    die "'$word' is not a valid value (off|on).\n";
}

package PureMessage::GroupAdminPermission::ReadWrite;
use strict;
use warnings;
use base 'PureMessage::GroupAdminPermission';

# Each object in this class is a row in the database holding
# a permission for a group/admin pair

__PACKAGE__->table(q{group_admin_read_write_permission});

sub word_to_bits {
    my ($pkg, $word) = @_;
    return 0 if $word eq 'off';
    return 2 if $word eq 'read';
    return 6 if $word eq 'write';
    die "'$word' is not a valid value (off|read|write).\n";
}

package PureMessage::Group::Permission;
use strict;
use warnings;

# This class is used for encapsulating a permission for passing
# to application code.  This object may not represent an object
# in the database in the case of default permissions.
#
# This class has no direct DB association, it is only used for
# encapsulating and displaying permissions

sub new {
    my ($class, $name, $value) = @_;
    die "Must supply a permission name" unless $name;
    die "Must supply a permission value" unless defined $value;
    my $self = { name => $name, value => $value };
    $self->{label} = { 0 => 'off',
                       1 => 'on',
                       2 => 'read',
                       4 => 'write',
                       6 => 'read/write',
                     };
    bless $self, $class;
    return $self;
}

sub name { $_[0]->{name} }

sub value { 
    my ($self) = @_;
    my $label = $self->{label}{$self->{value}};
    die "Invalid value $self->{value}" unless defined $label;
    return $label;
}

1;
