package PureMessage::Multifile::Sync;
use strict;
use warnings;
use PureMessage::Multifile::Constants qw($MULTIFILE_DIR $MULTIFILE_CONF);
use PureMessage::Multifile::DB;
use File::Path;
use ActiveState::File::Atomic;
use ActiveState::Config::Simple qw(config_parse_file);
use Digest::MD5 qw(md5_hex);

sub new {
    my ($proto) = @_;
    my $class = ref($proto) || $proto;
    my $self = {};
    bless($self, $class);
    return $self;
}

sub sync {
    my $self = shift;

    unless (-d $MULTIFILE_DIR) {
        mkpath $MULTIFILE_DIR or die "Can't mkpath $MULTIFILE_DIR: $!";
    }

    my $made_changes = 0;

    # Update multifiles from DB
    my $old_config = config_parse_file($MULTIFILE_CONF);
    my @multifiles = PureMessage::Multifile::DB->retrieve_all();
    $self->{_multifiles} = {};
    for my $mf (@multifiles) {
        next unless $mf->should_sync;
        my $id = $mf->id;
        my $local_dir = "$MULTIFILE_DIR/$id";
        unless (-d $local_dir) {
            $made_changes++;
            mkpath $local_dir or die "Can't mkpath $local_dir: $!";
        }

        # Record this multifile for writing the config later
        $self->{_multifiles}{$id} = {
            name => $mf->name,
            charset => $mf->charset,
        };

        delete $old_config->{multifile}{$id};

        my %old_files = map { s#.+/##; $_ => 1 } glob(qq{$local_dir/*});

        # Sync the group files
        my @group_files = $mf->group_files;
        for my $g (@group_files) {
            my $group_file = "$local_dir/" . $g->group_name;
            my $local_md5;
            if (-e $group_file) {
                open(my $rfh, $group_file) or die "Can't open $group_file: $!";
                my $ctx = Digest::MD5->new;
                $ctx->addfile($rfh);
                $local_md5 = $ctx->hexdigest;
                close $rfh;
            }

            delete $old_files{$g->group_name};

            next if $local_md5 and $local_md5 eq $g->checksum;

            my $as = ActiveState::File::Atomic->new( $group_file,
                                                     writable => 1,
                                                     create   => 1,
                                                   );
            $as->commit_string($g->content);
            $made_changes++;
        }

        for my $group (keys %old_files) {
            unlink "$local_dir/$group";
            $made_changes++;
        }
    }

    for my $id (keys %{ $old_config->{multifile} }) {
        rmtree "$MULTIFILE_DIR/$id" or die "Failed to rmtree $MULTIFILE_DIR/$id: $!";
        $made_changes++;
    }

    $self->write_config if $made_changes;
    return $made_changes;
}

sub write_config {
    my $self = shift;

    my $as = ActiveState::File::Atomic->new( $MULTIFILE_CONF,
                                             writable => 1,
                                             create   => 1,
                                           );
    my $wfh = $as->tempfile;
    print $wfh <<EOT;
# Multifile local copy
# This file, and the files in the multifile directory
# should not be edited by hand.  All local changes
# will be overwritten on the next 'pmx-profile sync-from-db'

EOT

    my $synced = $self->{_multifiles};
    for my $id (sort keys %$synced) {
        my $mf = $synced->{$id};
        print $wfh <<EOT;
<multifile $id>
  name = '$mf->{name}'
  charset = '$mf->{charset}'
  source = multifile:$id
</multifile>
EOT
    }
    $as->commit_tempfile;
}

1;
