package PureMessage::User;
use strict;
use warnings;
use PureMessage::Install::OptionValidator qw(valid_email);
use PureMessage::Group::Log qw/activity_log/;
use PureMessage::GroupAdmin;
use PureMessage::GroupAdminPermission;
use Digest::SHA1 qw(sha1_hex);
use base 'PureMessage::DBI';
use base 'Exporter';
our @EXPORT_OK = qw/login_validator $LOGIN_ERROR/;

__PACKAGE__->table(q{pmx_user});
__PACKAGE__->columns(Primary => q{user_name});
__PACKAGE__->columns(All => qw/user_name passphrase email fullname locale super_user/);
__PACKAGE__->has_many(admin_groups => q{PureMessage::GroupAdmin});

our $LOGIN_ERROR;

# friendly wrapper to create a new user
sub add_user {
    my %opts = @_;
    die qq{Must supply a user_name!\n} unless $opts{user_name};
    die qq{Invalid user_name: $opts{user_name}\n}
        unless $opts{user_name} =~ /^[A-Za-z][\w.-]+$/;
    die qq{Must supply a passphrase!\n} unless $opts{passphrase};
    die qq{Must supply an email address!\n} unless $opts{email};
    my $user_name = lc($opts{user_name});

    eval { valid_email($opts{email}) };
    die qq{Invalid email address: $opts{email}\n} if $@;

    # Check if user already exists, and return nice message if so.
    eval { __PACKAGE__->load($user_name) };
    die "The '$opts{user_name}' user already exists.\n" if !$@;

    $opts{passphrase} = sha1_hex($opts{passphrase}) unless $opts{pre_hashed};
    my $user;
    my %params = (
        user_name => $user_name,
        passphrase => $opts{passphrase},
        email => $opts{email},
        fullname => $opts{fullname} || '',
        locale => $opts{locale} || '',
        super_user => $opts{super_user},
    );
    eval { $user = __PACKAGE__->create(\%params) };
    die qq{Error creating user $user_name: $@\n} if $@;

    if ($opts{super_user}) {
        PureMessage::GroupAdmin::add_group_admin('supergroup', $user_name);
    }

    return $user;
}

sub delete_user {
    my $user_name = shift;
    die qq{Must supply a user_name!\n} unless $user_name;
    $user_name = lc($user_name);
    my $user = __PACKAGE__->search(user_name => $user_name)->first();
    die qq{No users found with user_name $user_name!\n}
        unless $user;
    $user->delete();
}

sub groups {
    my ($self) = @_;
    if ($self->super_user) {
        return PureMessage::Group->retrieve_all;
    }
    else {
        return $self->admin_groups;
    }
}

sub validate {
    my ($self, $user_name, $pass) = @_;

    $self->clear_object_index;
    $user_name = lc($user_name);
    my $user = $self->search(user_name => $user_name)->first();
    unless ($user) {
        activity_log(undef, "Can't validate unknown user ($user_name)");
        return undef;
    }
    unless (sha1_hex($pass) eq $user->passphrase) {
        activity_log(undef, "Wrong password for user $user_name");
        return undef;
    }
    activity_log(undef, "User $user_name is validated");
    return $user;
}

sub login_validator {
    my $user;
    eval {
        $user = PureMessage::User->validate(@_);
    };
    $LOGIN_ERROR = $@;
    return $user->user_name if $user;
}

sub set_passphrase {
    my ($user, $pass) = @_;
    $user->passphrase(sha1_hex($pass));
    $user->update;
}

sub load {
    my ($class, $username) = @_;
    $username ||= '';
    $class->clear_object_index();
    my $user;
    eval {
        $user = $class->search(user_name => lc($username))->first();
    };
    if ($@ or !$user) {
        die $@ if $@ =~ /DBI connect/; # pass thru dbi errors
        activity_log(undef, "Error loading '$username': $@") if $@;
        die "'$username' is not a valid username.\n";
    }
    return $user;
}

sub set_permission {
    my ($self, %opts) = @_;
    my $ga = _fetch_group_admin($self, $opts{group});
    PureMessage::GroupAdminPermission::set_permission($ga, $opts{perm},
                                                      $opts{value});
}

sub get_permissions {
    my ($self, %opts) = @_;
    my $ga = _fetch_group_admin($self, $opts{group});

    my $perm_name = $opts{perm};
    if ($perm_name) {
        return PureMessage::GroupAdminPermission::get_permission($ga, $opts{perm});
    }

    return PureMessage::GroupAdminPermission::all_permissions($ga);
}

sub can_do {
    my ($self, %opts) = @_;
    my $ga = _fetch_group_admin($self, $opts{group});
    return PureMessage::GroupAdminPermission->can($ga, $opts{perm});
}

sub can_read {
    my ($self, %opts) = @_;
    my $ga = _fetch_group_admin($self, $opts{group});
    return PureMessage::GroupAdminPermission->can_read($ga, $opts{perm});
}

sub can_write {
    my ($self, %opts) = @_;
    my $ga = _fetch_group_admin($self, $opts{group});
    return PureMessage::GroupAdminPermission->can_write($ga, $opts{perm});
}

sub _fetch_group_admin {
    my ($self, $groupname) = @_;
    $groupname = 'supergroup' if $self->super_user;
    die "group is a mandatory option!\n" unless $groupname;

    my $username = $self->user_name;
    my $ga = PureMessage::GroupAdmin->search(user_name => $username,
                                             group_name => $groupname)->first;
    die "Could not find group/user pair for $groupname/$username\n" 
        unless $ga;
    return $ga;
}

sub any_group_can {
    my ($self, @perms) = @_;
    my @groups = $self->groups;
    my %perms;
    my $user_name = $self->user_name;
    for my $g (@groups) {
        my $ga = _fetch_group_admin($self, $g->group_name);
        for my $p (@perms) {
            next if $perms{$p};
            $perms{$p} = PureMessage::GroupAdminPermission->can($ga, $p);
        }
    }

    if (@perms == 1) {
        return $perms{$perms[0]};
    }

    return \%perms;
}

sub reset_passphrase {
    my %opts = @_;
    die "Must supply a username to reset!\n" unless $opts{user_name};
    die "Must supply a new passphrase!\n" unless $opts{passphrase};
    my $user_name = lc $opts{user_name};
    __PACKAGE__->clear_object_index();
    my ($user) = __PACKAGE__->search(user_name => $user_name);
    die "'$user_name' is not a valid user!\n" unless $user;
    $user->set_passphrase($opts{passphrase});
    activity_log($user_name, 'Reset password');
}

1;
